@@ -1,4 +1,4 @@
-# Copyright 2004-2019 Davide Alberani <da@erlug.linux.it>
+# Copyright 2004-2020 Davide Alberani <da@erlug.linux.it>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,37 +21,33 @@ the IMDb web pages, or a SQL database.
 """
 
 from __future__ import absolute_import, division, print_function, unicode_literals
-
+from imdb.version import __version__
 
 __all__ = ['IMDb', 'IMDbError', 'Movie', 'Person', 'Character', 'Company',
            'available_access_systems']
-__version__ = VERSION = '6.8'
+
+VERSION = __version__
 
 
-import logging
 import os
 import sys
 from pkgutil import find_loader
 from types import MethodType, FunctionType
 
-import imdb._logging
+from imdb._logging import imdbpyLogger as _imdb_logger
 from imdb._exceptions import IMDbDataAccessError, IMDbError
 from imdb import Character, Company, Movie, Person
 from imdb.utils import build_company_name, build_name, build_title
 
-
 PY2 = sys.hexversion < 0x3000000
 
-
 if PY2:
     import ConfigParser as configparser
 else:
     import configparser
 
 
-_imdb_logger = logging.getLogger('imdbpy')
-_aux_logger = logging.getLogger('imdbpy.aux')
-
+_aux_logger = _imdb_logger.getChild('aux')
 
 # URLs of the main pages for movies, persons, characters and queries.
 imdbURL_base = 'https://www.imdb.com/'
@@ -76,13 +72,15 @@ imdbURL_company_base = '%scompany/' % imdbURL_base
 # http://www.imdb.com/company/co%s/
 imdbURL_company_main = imdbURL_company_base + 'co%s/'
 # http://www.imdb.com/keyword/%s/
-imdbURL_keyword_main = imdbURL_base + 'keyword/%s/'
+imdbURL_keyword_main = imdbURL_base + 'search/keyword/?keywords=%s'
 # http://www.imdb.com/chart/top
 imdbURL_top250 = imdbURL_base + 'chart/top'
 # http://www.imdb.com/chart/bottom
 imdbURL_bottom100 = imdbURL_base + 'chart/bottom'
 # http://www.imdb.com/find?%s
 imdbURL_find = imdbURL_base + 'find?%s'
+# http://www.imdb.com/list/
+imdbURL_list_base = imdbURL_base + 'list/'
 
 # Name of the configuration file.
 confFileName = 'imdbpy.cfg'
@@ -292,15 +290,25 @@ class IMDbBase:
         # http://www.imdb.com/company/co%s/
         imdbURL_company_main = imdbURL_company_base + 'co%s/'
         # http://www.imdb.com/keyword/%s/
-        imdbURL_keyword_main = imdbURL_base + 'keyword/%s/'
+        imdbURL_keyword_main = imdbURL_base + '/search/keyword?keywords=%s'
         # http://www.imdb.com/chart/top
         imdbURL_top250 = imdbURL_base + 'chart/top'
         # http://www.imdb.com/chart/bottom
         imdbURL_bottom100 = imdbURL_base + 'chart/bottom'
+        # http://www.imdb.com/chart/moviemeter
+        imdbURL_moviemeter100 = imdbURL_base + 'chart/moviemeter'
+        # http://www.imdb.com/chart/tvmeter
+        imdbURL_tvmeter100 = imdbURL_base + 'chart/tvmeter'
+        # http://www.imdb.com/chart/toptv
+        imdbURL_toptv250 = imdbURL_base + 'chart/toptv'
+        # https://www.imdb.com/india/top-rated-indian-movies
+        imdbURL_topindian250 = imdbURL_base + 'india/top-rated-indian-movies'
         # http://www.imdb.com/find?%s
         imdbURL_find = imdbURL_base + 'find?%s'
         # http://www.imdb.com/search/title?%s
         imdbURL_search_movie_advanced = imdbURL_base + 'search/title/?%s'
+        # http://www.imdb.com/list/
+        imdbURL_list_base = imdbURL_base + 'list/'
         self.urls = dict(
             movie_base=imdbURL_movie_base,
             movie_main=imdbURL_movie_main,
@@ -313,7 +321,12 @@ class IMDbBase:
             keyword_main=imdbURL_keyword_main,
             top250=imdbURL_top250,
             bottom100=imdbURL_bottom100,
+            moviemeter100=imdbURL_moviemeter100,
+            tvmeter100=imdbURL_tvmeter100,
+            toptv250=imdbURL_toptv250,
+            topindian250=imdbURL_topindian250,
             find=imdbURL_find,
+            movie_list=imdbURL_list_base,
             search_movie_advanced=imdbURL_search_movie_advanced)
 
     def _normalize_movieID(self, movieID):
@@ -429,6 +442,19 @@ class IMDbBase:
                 data=md, modFunct=self._defModFunct,
                 accessSystem=self.accessSystem) for mi, md in res][:results]
 
+    def _get_movie_list(self, list_, results):
+        """Return a list of tuples (movieID, {movieData})"""
+        # XXX: for the real implementation, see the method of the
+        #      subclass, somewhere under the imdb.parser package.
+        raise NotImplementedError('override this method')
+
+    def get_movie_list(self, list_, results=None):
+        """Return a list of Movie objects for a list id as input """
+        res = self._get_movie_list(list_, results)
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res][:results]
+
     def _search_movie_advanced(self, title=None, adult=None, results=None, sort=None, sort_dir=None):
         """Return a list of tuples (movieID, {movieData})"""
         # XXX: for the real implementation, see the method of the
@@ -603,13 +629,13 @@ class IMDbBase:
             results = 100
         return self._search_keyword(keyword, results)
 
-    def _get_keyword(self, keyword, results):
+    def _get_keyword(self, keyword, results, page):
         """Return a list of tuples (movieID, {movieData})"""
         # XXX: for the real implementation, see the method of the
         #      subclass, somewhere under the imdb.parser package.
         raise NotImplementedError('override this method')
 
-    def get_keyword(self, keyword, results=None):
+    def get_keyword(self, keyword, results=None, page=None):
         """Return a list of movies for the given keyword."""
         if results is None:
             results = self._keywordsResults
@@ -617,7 +643,7 @@ class IMDbBase:
             results = int(results)
         except (ValueError, OverflowError):
             results = 100
-        res = self._get_keyword(keyword, results)
+        res = self._get_keyword(keyword, results, page)
         return [Movie.Movie(movieID=self._get_real_movieID(mi),
                 data=md, modFunct=self._defModFunct,
                 accessSystem=self.accessSystem) for mi, md in res][:results]
@@ -644,6 +670,35 @@ class IMDbBase:
                 data=md, modFunct=self._defModFunct,
                 accessSystem=self.accessSystem) for mi, md in res]
 
+    def get_top250_tv(self):
+        """Return the list of the top 250 tv shows."""
+        res = self._get_top_bottom_movies('toptv')
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res]
+
+    def get_popular100_movies(self):
+        """Return the list of the 100 most popular movies."""
+        res = self._get_top_bottom_movies('moviemeter')
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res]
+
+    def get_popular100_tv(self):
+        """Return the list of the 100 most popular tv shows."""
+        res = self._get_top_bottom_movies('tvmeter')
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res]
+
+    def get_top250_indian_movies(self):
+        """Return the list of the top 250 indian movies."""
+        res = self._get_top_bottom_movies('topindian250')
+        return [Movie.Movie(movieID=self._get_real_movieID(mi),
+                data=md, modFunct=self._defModFunct,
+                accessSystem=self.accessSystem) for mi, md in res]
+
+
     def new_movie(self, *arguments, **keywords):
         """Return a Movie object."""
         # XXX: not really useful...
@@ -758,6 +813,68 @@ class IMDbBase:
                 mop.update_charactersRefs(ret['charactersRefs'])
         mop.set_data(res, override=0)
 
+    def update_series_seasons(self, mop, season_nums, override=0):
+        """Given a Movie object with only retrieve the season data.
+
+        season_nums is the list of the specific seasons to retrieve.
+
+        If override is set, the information are retrieved and updated
+        even if they're already in the object."""
+        mopID = None
+        if isinstance(mop, Movie.Movie):
+            mopID = mop.movieID
+        else:
+            raise IMDbError('object ' + repr(mop) + ' is not a Movie instance')
+        if mopID is None:
+            raise IMDbDataAccessError('supplied object has null movieID, personID or companyID')
+        if mop.accessSystem == self.accessSystem:
+            aSystem = self
+        else:
+            aSystem = IMDb(mop.accessSystem)
+
+        info = 'episodes'
+
+        res = {}
+
+        if info in mop.current_info and not override:
+            return
+        _imdb_logger.debug('retrieving "%s" info set', info)
+        try:
+            method = getattr(aSystem, 'get_movie_episodes')
+        except AttributeError:
+            _imdb_logger.error('unknown information set "%s"', info)
+            # Keeps going.
+            method = lambda *x: {}
+        try:
+            ret = method(mopID, season_nums)
+        except Exception:
+            _imdb_logger.critical(
+                'caught an exception retrieving or parsing "%s" info set'
+                ' for mopID "%s" (accessSystem: %s)',
+                info, mopID, mop.accessSystem, exc_info=True
+            )
+            ret = {}
+            # If requested by the user, reraise the exception.
+            if self._reraise_exceptions:
+                raise
+        keys = None
+        if 'data' in ret:
+            res.update(ret['data'])
+            if isinstance(ret['data'], dict):
+                keys = list(ret['data'].keys())
+        if 'info sets' in ret:
+            for ri in ret['info sets']:
+                mop.add_to_current_info(ri, keys, mainInfoset=info)
+        else:
+            mop.add_to_current_info(info, keys)
+        if 'titlesRefs' in ret:
+            mop.update_titlesRefs(ret['titlesRefs'])
+        if 'namesRefs' in ret:
+            mop.update_namesRefs(ret['namesRefs'])
+        if 'charactersRefs' in ret:
+            mop.update_charactersRefs(ret['charactersRefs'])
+        mop.set_data(res, override=0)
+
     def get_imdbMovieID(self, movieID):
         """Translate a movieID in an imdbID (the ID used by the IMDb
         web server); must be overridden by the subclass."""
