@@ -18,6 +18,8 @@
 This module provides the logging facilities used by the imdb package.
 """
 
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 import logging
 
 
@@ -44,5 +46,5 @@ def setLevel(level):
     """Set logging level for the main logger."""
     level = level.lower().strip()
     imdbpyLogger.setLevel(LEVELS.get(level, logging.NOTSET))
-    imdbpyLogger.log(imdbpyLogger.level, 'set logging threshold to "%s"',
+    imdbpyLogger.log(logging.INFO, 'set logging threshold to "%s"',
                      logging.getLevelName(imdbpyLogger.level))
